/**
  *  \file client/widgets/partstorage.cc
  *  \brief Ship Build Screen Part Storage and Image Display
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/partstorage.h"
#include "cpluslib/format.h"
#include "gfx/gfxcontext.h"
#include "ui/layout.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "util/misc.h"

/************************** WPartStorageDisplay **************************/

WPartStorageDisplay::WPartStorageDisplay()
    : UIWidget(), n(0)
{ }

void
WPartStorageDisplay::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setSolidBackground();
    ctx.setColor(UIColor::tc_Static);
    outTextF(ctx, getExtent().x, getExtent().y, getExtent().w,
             _("In storage:"));
    outTextF(ctx, getExtent().x, getExtent().y + font_heights[FONT_NORMAL], getExtent().w,
             format(_("%d pcs."), numToString(n)));
    can.update(getExtent());
}

void
WPartStorageDisplay::getLayoutInfo(LayoutInfo& info)
{
    info.min_size = info.pref_size = font_sizes[FONT_NORMAL].scaledBy(7, 2);
    info.flags = 0;
}

void
WPartStorageDisplay::setCount(int n)
{
    if (n != this->n) {
        this->n = n;
        drawWidget(false);
    }
}


WPartStorageGroup::WPartStorageGroup(bool withBuild)
    : UIGroup(UIVBoxLayout::instance5)
{
    image = &h.add(new UIImageWidget(105+2, 93+2 /* FIXME! Make this variable? */));
    psd = &h.add(new WPartStorageDisplay());

    add(*image);
    add(*psd);
    if (withBuild) {
        add(h.add(new UIButton(_("Spc-Build"), ' ')));
    } else {
        add(h.add(new UISpacer()));
    }
}

WPartStorageGroup::~WPartStorageGroup()
{ }
