/**
  *  \file client/widgets/setselect.h
  *  \brief Class WPlayerSetSelector
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_SETSELECT_H
#define PCC_V2_CLIENT_WIDGETS_SETSELECT_H

#include "ui/listbox.h"
#include "game/playerset.h"

/** Player set selector. Displays a list of players with optional
    information, and lets the user choose a subset.

    Use for
    - unpack
    - sweep
    - message receiver

    FIXME: this could use a little more chrome */
class WPlayerSetSelector : public UIListbox {
    int max;
    GPlayerArray<string_t> info;
    GPlayerSet selectable_players;
    GPlayerSet selected_players;
 public:
    WPlayerSetSelector(int id, int max, int width);
    ~WPlayerSetSelector();

    void setSelectedPlayers(GPlayerSet players);
    void setSelectablePlayers(GPlayerSet players);
    void setInfo(int pid, string_t info);
    GPlayerSet getSelectedPlayers() const;

    void togglePlayers(GPlayerSet players);
    void togglePlayer(int player);
    void toggleUniversal();
    void scrollToPlayer(int player);
    int getCurrentPlayer() const;

    // UIListbox:
    bool isAccessible(int pos);
    void drawPart(GfxCanvas& can, int from, int to);
    void onItemClick(int item, int x);

    bool handleEvent(const UIEvent& e, bool second_pass);
};

inline GPlayerSet
WPlayerSetSelector::getSelectedPlayers() const
{
    return selected_players;
}

#endif
