// \ingroup FLAK Structures
/**
  *  \file flak/flakdefs.h
  *  \brief Structure Definitions (Structures)
  *  \ingroup FLAK Structures
  *
  *  Automatically generated by scripts/iogen.pl from flak/flakdefs.in
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#ifndef PCC_V2_FLAK_FLAKDEFS_H
#define PCC_V2_FLAK_FLAKDEFS_H

#include "cpluslib/types.h"
#include "io/io.h"

/** FLAK Version Number string. */
#define FLAK_VERSION    "0.0.1"
/** FLAK Copyright message. Displayed at startup. */
#define FLAK_COPYRIGHT  "(c) 2003-2010 Stefan Reuther and others"
/** FLAK Support Email address. Displayed on help screen. */
#define FLAK_MAIL       "flak@phost.de"

#define FLAK_ALGORITHM 20060624
// #define FLAK_ALGORITHM 20060531
// #define FLAK_ALGORITHM 20060228
// #define FLAK_ALGORITHM 0

/** FLAK Manifest constants. */
enum {
    /* Manifest constants for game rules */
    FLAK_MAX_BEAMS                        = 20,     ///< Maximum number of beams per object.
    FLAK_MAX_TORPS                        = 20,     ///< Maximum number of torpedo launchers per object.
    FLAK_MAX_BAYS                         = 50,     ///< Maximum number of fighter bays per object.
    FLAK_NUM_OWNERS                       = 12,     ///< Maximum value for TFlakShip::player.

    /* We choose a new enemy every this many ticks. */
    FLAK_CHOOSE_ENEMY_TIME                = 30,     ///< Interval between choosing enemies.

    /* Targeting */
    FLAK_DIFF_OFFSET                      = 100,    ///< Offset for targeting rating difference.
    FLAK_DIVISOR_IF_SMALLER               = 100,    ///< Targeting divisor if we're smaller than the target.
#if FLAK_ALGORITHM >= 20060228
    FLAK_DIVISOR_IF_BIGGER                = 250,    ///< Targeting divisor if we're bigger than the target.
    FLAK_DIVISOR_SAME_ENEMY_BONUS         = 150,    ///< Targeting divisor bonus for keeping same target.
#else
    FLAK_DIVISOR_IF_BIGGER                = 150,    ///< Targeting divisor if we're bigger than the target.
    FLAK_DIVISOR_SAME_ENEMY_BONUS         = 50,     ///< Targeting divisor bonus for keeping same target.
#endif
    FLAK_MULTIPLIER_MIN                   = 50,     ///< Minimum targeting multiplier.

    /* Torpedo firepower limitation */
#if FLAK_ALGORITHM >= 20060228
    FLAK_TORP_LIMIT_FACTOR                = 120,    ///< Safety factor for torpedo limit.
#else
    FLAK_TORP_LIMIT_FACTOR                = 150,    ///< Safety factor for torpedo limit.
#endif

    /* Compensation */
    FLAK_COMPENSATION_LIMIT               = 2,      ///< Maximum compensation bonus.
    FLAK_COMPENSATION_DIVISOR             = 1000,   ///< Compensation formula offset.

    /* Combat */
    FLAK_TORP_MOVEMENT_SPEED              = 1000,   ///< Torpedo movement speed, meters per tick.
    FLAK_FIGHTER_INTERCEPT_RANGE          = 128,    ///< Fighter intercept range, meters.

    /* MaxFightersLaunched */
    FLAK_MFL_SCALE                        = 2,      ///< MaxFightersLaunched per bay.
    FLAK_MAXIMUM_MFL                      = 50,     ///< Maximum fighters launched.
    FLAK_MINIMUM_MFL                      = 1       ///< Minimum number of fighters allowed for a ship.
};

/** FLAK flags. Values for TFlakShip::flags. */
enum {
    flak_IsPlanet = 1                   ///< Set if this is a planet, not a ship.
};

/** FLAK Ship Data. These values are constant throughout a fight. */
struct TFlakShip {
    char    name[20];           ///< Name.
    int16   damage_init;                ///< Initial damage.
    int16   crew_init;                  ///< Initial crew.
    int16   id;                         ///< Id number.
    int16   player;                     ///< Player number.
    int16   hull;                       ///< Hull type.
    int16   level;                      ///< Experience level.
    int16   beam_count;                 ///< Number of beams.
    int16   beam_type;                  ///< Beam type.
    int16   torp_lcount;                ///< Number of torpedo launchers.
    int16   torp_count;                 ///< Number of torpedoes.
    int16   torp_type;                  ///< Torpedo type.
    int16   bay_count;                  ///< Number of fighter bays.
    int16   fighter_count;              ///< Number of fighters.
    int16   mass;                       ///< Combat mass.
    int16   shield_init;                ///< Initial shields.
    int16   max_fighters_launched;      ///< Maximum number of simultaneously launched fighters allowed.
    int32   rating;                     ///< Targeting rating.
    int16   compensation_rating;        ///< Compensation rating.
    int16   flags;                      ///< Additional flags.
    // I don't like this `mutable' here, but it saves quite some
    // headache in flak.cc where this structure is used as a `const'
    mutable int16 ending_status;
    enum {
       size  = 62,      ///< Size of this structure in DOS
       count = 21       ///< Number of members
    };
    static const TStructureParser parser[];
};

/** FLAK Fleet Data. */
struct TFlakFleet {
    int16   player;                     ///< Player number.
    int16   first_ship;                 ///< Index of first ship (0-based).
    int16   num_ships;                  ///< Number of ships.
    int16   speed;                      ///< Speed.
    int32   att_list_pointer;           ///< Index of first attack list entry (0-based).
    int32   att_list_size;              ///< Number of attack list entries.
    int32   x_init, y_init;             ///< Initial position (combat coordinates, meters).
    enum {
       size  = 24,      ///< Size of this structure in DOS
       count = 8       ///< Number of members
    };
    static const TStructureParser parser[];
};

/** FLAK Battle Data. */
struct TFlakBattle {
    int32   this_size;                  ///< Total size in file.
    int16   x, y;                       ///< Location in universe (starchart coordinates, ly).
    int32   seed;                       ///< Random number seed.
    int32   total_time;                 ///< Total time required to resolve this fight.
    int32   ambient_flags;              ///< Ambient flags.

    int32   num_fleets;                 ///< Number of fleets.
    int32   fleet_entry_size;           ///< Size of a fleet entry. Equals TFlakFleet::size.
    int32   fleet_ptr;                  ///< Position of first fleet, relative to start of this record.

    int32   num_ships;                  ///< Number of ships.
    int32   ship_entry_size;            ///< Size of a ship entry. Equals TFlakShip::size.
    int32   ship_ptr;                   ///< Position of first ship, relative to start of this record.

    int32   num_att_list_entries;       ///< Number of attack list entries.
    int32   att_list_entry_size;        ///< Size of an attack list entry. Set to 4 (=2 int16's).
    int32   att_list_ptr;               ///< Position of first attack list entry, relative to start of this record.
    enum {
       size  = 56,      ///< Size of this structure in DOS
       count = 15       ///< Number of members
    };
    static const TStructureParser parser[];
};

/** FLAK File Header. A FLAK file starts with this record, and is
    followed by a handful of TFlakBattle records. */
struct TFlakHeader {
    char    magic[8];                   ///< Magic number.
    int16   filefmt_version;            ///< File format version.
    int16   player;                     ///< Player number. Addressee of this file.
    int16   turn;                       ///< Turn number.
    int16   num_battles;                ///< Number of battles.
    char    timestamp[18];              ///< Timestamp.
    int32   reserved;                   ///< Reserved for future expansion.
    enum {
       size  = 38,      ///< Size of this structure in DOS
       count = 7       ///< Number of members
    };
    static const TStructureParser parser[];
};

#endif
