/**
  *  \file game/actions/planetstructs.h
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ACTIONS_PLANETSTRUCTS_H
#define PCC_V2_GAME_ACTIONS_PLANETSTRUCTS_H

#include "game/planet.h"
#include "game/actions/cargocost.h"

//! Building structures on a planet.
class GPlanetBuildStructuresAction : public GCargoCostTransaction {
    enum { MAX = str_BaseDefense + 1 };
    struct Data {
        int   order;            ///< Amount selected by user.
        int   max;              ///< Maximum amount allowed by rules.
        int   min;              ///< Minimum amount allowed by rules (undo).
    };
    Data      data[MAX];
    GPlanet&  the_planet;

    SignalConnection conn;
    void update();

 protected:
    void commit();

 public:
    GPlanetBuildStructuresAction(GPlanet& the_planet, GCargoContainer& container);
    ~GPlanetBuildStructuresAction();

    void setUndoInformation(const GPlanet& dis_planet);

    int addStructures(TPlanetaryStructure ps, int count, bool partial);
    int addStructuresLimitCash(TPlanetaryStructure ps, int count);

    GPlanet& getPlanet();
    int getMin(TPlanetaryStructure ps) const;
    int getMax(TPlanetaryStructure ps) const;
    int getAmount(TPlanetaryStructure ps) const;

    void doStandardAutoBuild();

    Signal0 sig_changed;
};

/** Get planet we're working on. */
inline GPlanet&
GPlanetBuildStructuresAction::getPlanet()
{
    return the_planet;
}

/** Get minimum amount for a structure. Structures cannot be destroyed. */
inline int
GPlanetBuildStructuresAction::getMin(TPlanetaryStructure ps) const
{
    return data[ps].min;
}
/** Get maximum amount for a structure. */
inline int
GPlanetBuildStructuresAction::getMax(TPlanetaryStructure ps) const
{
    return data[ps].max;
}
/** Get current ordered amount for a structure. */
inline int
GPlanetBuildStructuresAction::getAmount(TPlanetaryStructure ps) const
{
    return data[ps].order;
}

#endif
