/**
  *  \file game/pref.h
  *  \brief User Preferences
  *
  *  This file contains the user preferences (in contrast to
  *  game/config.h, which contains the game configuration).
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_PREF_H
#define PCC_V2_GAME_PREF_H

#include "util/conf.h"

class GUserPreferences : public Config {
 public:
    GUserPreferences();
    void assignDefaults();

    // Game-related options:
    ConfigIntOption CollapseOldMessages;
    ConfigIntOption RewrapMessages;
    ConfigIntOption InstantBattleResult;

    // Sound options
    ConfigIntOption Sound16Bits;
    ConfigIntOption SoundEnabled;
    ConfigIntOption SoundFrequency;
    ConfigIntOption SoundHeadphone;
    ConfigIntOption SoundReverse;
    ConfigIntOption SoundStereo;

    // Starchart options
    ConfigIntOption ChartAnimThreshold;
    ConfigIntOption ChartMouseStickiness;
    ConfigIntOption ChartScannerWarpWells;

    // Alliance
    ConfigIntOption TeamAutoSync;

    // Preference
    ConfigIntOption DisplayThousandSep;
    ConfigIntOption DisplayClans;

    // Export
    ConfigStringOption ExportShipFields;
    ConfigStringOption ExportPlanetFields;
};

GUserPreferences& getUserPreferences();

void createUserPreferencesDirectory();

extern const struct ModuleInitializer mod_pref[];

#endif
