/**
  *  \file int/export/expconfig.cc
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <stdexcept>
#include "int/export/expconfig.h"
#include "util/nls.h"
#include "cpluslib/format.h"

IntExportConfigParser::IntExportConfigParser(IntExportFieldList& fieldList)
    : m_fieldList(fieldList),
      m_outputFormat(),
      m_charsetIndex(),
      m_outputFormatSet(false),
      m_charsetIndexSet(false)
{
    configure("export", true);
}

void
IntExportConfigParser::assign(int line_id, string_t name, string_t value, string_t /*line*/)
{
    try {
        if (strCaseCmp(name, "Fields") == 0) {
            m_fieldList.addList(value);
        } else if (strCaseCmp(name, "Format") == 0) {
            if (!parseOutputFormatType(value, m_outputFormat)) {
                // This is the same error message as in c2export
                throw std::runtime_error(_("invalid output format specified"));
            }
            m_outputFormatSet = true;
        } else if (strCaseCmp(name, "Charset") == 0) {
            if (CharacterSet::isCharsetNameUtf8(value)) {
                m_charsetIndex = 0;
                m_charsetIndexSet = true;
            } else {
                for (int i = 0, n = CharacterSet::getNumCharsets(); i < n; ++i) {
                    if (strCaseCmp(value, CharacterSet::getCharset(i).getName()) == 0) {
                        m_charsetIndex = i+1;
                        m_charsetIndexSet = true;
                        break;
                    }
                }
            }
            if (!m_charsetIndexSet) {
                throw std::runtime_error(_("the specified character set is not known"));
            }
        } else {
            // nix
        }
    }
    catch (std::exception& e) {
        throw std::runtime_error(string_t(format(_("%s (in line %d)"), e.what(), line_id)).c_str());
    }
}


bool
IntExportConfigParser::getOutputFormat(IntExportOutputType& result) const
{
    if (m_outputFormatSet) {
        result = m_outputFormat;
        return true;
    } else {
        return false;
    }
}

bool
IntExportConfigParser::getCharsetIndex(int& result) const
{
    if (m_charsetIndexSet) {
        result = m_charsetIndex;
        return true;
    } else {
        return false;
    }
}
